package com.miya.skin_lib.utils;

import android.app.Activity;
import android.content.Context;
import android.content.res.ColorStateList;
import android.content.res.Resources;
import android.content.res.TypedArray;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.support.v4.content.res.ResourcesCompat;
import android.text.TextUtils;


/**
 * 修改主题信息
 */
public class SkinThemeUtils {

    /**
     * 以下的几个模式的资源，取决于自己应用要用什么资源，和当前版本中主要用什么资源，可以根据项目具体使用来改改
     */
    private static int[] APPCOMPAT_COLOR_PRIMARY_DARK_ATTRS = {
            android.R.attr.colorPrimaryDark
    };
    private static int[] STATUSBAR_COLOR_ATTRS = {android.R.attr.statusBarColor, android.R.attr
            .navigationBarColor
    };

    /**
     * 获得theme中的属性中定义的 资源id
     *
     * @param context
     * @param attrs
     * @return
     */
    public static int[] getResId(Context context, int[] attrs) {
        int[] resIds = new int[attrs.length];
        TypedArray a = context.obtainStyledAttributes(attrs);
        for (int i = 0; i < attrs.length; i++) {
            resIds[i] = a.getResourceId(i, 0);
        }
        a.recycle();
        return resIds;
    }

    public static void updateStatusBarColor(Activity activity) {
        if (activity == null) {
            return;
        }
        //5.0以上才能修改
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            return;
        }
        //获得 statusBarColor 与 nanavigationBarColor (状态栏颜色)
        //当与 colorPrimaryDark  不同时 以statusBarColor为准
        int[] resIds = getResId(activity, STATUSBAR_COLOR_ATTRS);
        int statusBarColorResId = resIds[0];
        int navigationBarColor = resIds[1];

        //如果直接在style中写入固定颜色值(而不是 @color/XXX ) 获得0
        if (statusBarColorResId != 0) {
            int color = SkinResources.getInstance().getColor(statusBarColorResId);
            activity.getWindow().setStatusBarColor(color);
        } else {
            //获得 colorPrimaryDark
            int colorPrimaryDarkResId = getResId(activity, APPCOMPAT_COLOR_PRIMARY_DARK_ATTRS)[0];
            if (colorPrimaryDarkResId != 0) {
                int color = SkinResources.getInstance().getColor(colorPrimaryDarkResId);
                activity.getWindow().setStatusBarColor(color);
            }
        }
        if (navigationBarColor != 0) {
            int color = SkinResources.getInstance().getColor
                    (navigationBarColor);
            activity.getWindow().setNavigationBarColor(color);
        }
    }

    public static class SkinResources {


        //皮肤包的包名
        private String skinPkgName;
        //是否使用默认的皮肤，用原始的内容
        private boolean isDefaultSkin = true;

        //app原始的resource
        private Resources appResources;
        //皮肤包的resources
        private Resources skinResources;

        private static volatile SkinResources instance;

        private SkinResources(Context context) {
            appResources = context.getResources();
        }

        public static void init(Context context) {
            if (null == instance) {
                synchronized (SkinResources.class) {
                    if (null == instance) {
                        instance = new SkinResources(context);
                    }
                }
            }
        }

        public static SkinResources getInstance() {
            return instance;
        }

        public void reset() {
            skinResources = null;
            skinPkgName = null;
            isDefaultSkin = true;
        }

        public void applySkin(Resources resources, String pkgName) {
            skinResources = resources;
            skinPkgName = pkgName;
            //是否使用默认皮肤
            isDefaultSkin = TextUtils.isEmpty(pkgName) || resources == null;
        }

        /**
         * 1. 通过原始app中的resId获取到自己的名字（R.color.icon_up，获取到的就是icon_up）和类型
         * 2. 根据名字和类型获取皮肤包中的ID
         */
        public int getIdentifier(int resId) {
            if (isDefaultSkin) {
                return resId;
            }
            //获取名称，如：icon_up
            String resName = appResources.getResourceEntryName(resId);
            //获取类型，如：drawable
            String resType = appResources.getResourceTypeName(resId);
            int skinId = skinResources.getIdentifier(resName, resType, skinPkgName);
            return skinId;
        }

        /**
         * 输入主app的id，获取皮肤包中的
         */
        public int getColor(int resId) {
            if (isDefaultSkin) {
                return appResources.getColor(resId);
            }
            int skinId = getIdentifier(resId);
            if (skinId == 0) {
                return appResources.getColor(resId);
            }
            return skinResources.getColor(skinId);
        }

        public Drawable getDrawable(int resId) {
            if (isDefaultSkin) {
                return ResourcesCompat.getDrawable(appResources, resId, null);
            }
            int skinId = getIdentifier(resId);
            if (skinId == 0) {
                return ResourcesCompat.getDrawable(appResources, resId, null);
            }
            return ResourcesCompat.getDrawable(skinResources, skinId, null);
        }

        public ColorStateList getColorStateList(int resId) {
            if (isDefaultSkin) {
                return ResourcesCompat.getColorStateList(appResources, resId, null);
            }
            int skinId = getIdentifier(resId);
            if (skinId == 0) {
                return ResourcesCompat.getColorStateList(appResources, resId, null);
            }
            return ResourcesCompat.getColorStateList(skinResources, skinId, null);
        }


        /**
         * 可能是Color 也可能是drawable
         *
         * @return
         */
        public Object getBackground(int resId) {
            String resourceTypeName = appResources.getResourceTypeName(resId);
            if ("color".equals(resourceTypeName)) {
                return getColor(resId);
            } else {
                // drawable
                return getDrawable(resId);
            }
        }

    }
}
